##############################################################################
#
# Copyright (c) 2004-2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit tests for the testrunner's subunit integration.
"""


import sys
import unittest
import formatter
from StringIO import StringIO


class FormatterOptions:
    """Simple options class for use with formatter."""
    verbose=False


class TestSubunitTracebackPrinting(unittest.TestCase):

    def makeByteStringFailure(self, text, encoding):
        try:
            # Note that this deliberately throws a string of bytes instead
            # of a unicode object.  This simulates errors thrown by
            # utf8-encoded doctests.
            bytestr = text.encode(encoding)
            self.fail(bytestr)
        except self.failureException:
            return sys.exc_info()

    def setUp(self):
        options = FormatterOptions()

        self.output = StringIO()
        self.subunit_formatter = formatter.SubunitOutputFormatter(
            options, stream=self.output)

    def test_print_failure_containing_utf8_bytestrings(self):
        exc_info = self.makeByteStringFailure(unichr(6514), 'utf8')
        self.subunit_formatter.test_failure(self, 0, exc_info)
        assert "AssertionError: \xe1\xa5\xb2" in self.output.getvalue()

    def test_print_error_containing_utf8_bytestrings(self):
        exc_info = self.makeByteStringFailure(unichr(6514), 'utf8')
        self.subunit_formatter.test_error(self, 0, exc_info)
        assert "AssertionError: \xe1\xa5\xb2" in self.output.getvalue()

    def test_print_failure_containing_latin1_bytestrings(self):
        exc_info = self.makeByteStringFailure(unichr(241), 'latin1')
        self.subunit_formatter.test_failure(self, 0, exc_info)
        self.assertIn("\xef\xbf\xbd", self.output.getvalue())


class TestSubunitStreamReporting(unittest.TestCase):
    """Test capture of stdout and stderr.

    The testrunner sets up fake I/O streams for the tests to use for
    sys.stdout and sys.stderr.  Anything written to those streams is
    captured and added as part of the subunit multi-part details output.
    """
    def setFakeStreams(self):
        sys.stdout = StringIO()
        sys.stderr = StringIO()

    def restoreStreams(self):
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__

    def makeExcInfo(self):
        try:
            assert False
        except:
            return sys.exc_info()

    def setUp(self):
        class FormatterOptions:
            verbose = False
        options = FormatterOptions()

        self.output = StringIO()
        self.setFakeStreams()
        self.subunit_formatter = formatter.SubunitOutputFormatter(
            options, stream=self.output)
        #self.subunit_formatter._set_stream_positions()

    def tearDown(self):
        self.restoreStreams()

    def test_stream_success(self):
        sys.stdout.write("Output written to stdout\n")
        sys.stderr.write("Output written to stderr\n")
        fake_test = formatter.FakeTest('stdout_test')
        self.subunit_formatter.test_success(fake_test, 10)
        self.restoreStreams()
        self.assertIn('STDOUT:', self.output.getvalue())
        self.assertIn('STDERR:', self.output.getvalue())


    def test_stream_error(self):
        sys.stdout.write("Output written to stdout\n")
        sys.stderr.write("Output written to stderr\n")
        fake_test = formatter.FakeTest('error_test')
        exc_info = self.makeExcInfo()
        self.subunit_formatter.test_error(fake_test, 10, exc_info)
        self.restoreStreams()
        self.assertIn('STDOUT:', self.output.getvalue())
        self.assertIn('STDERR:', self.output.getvalue())

    def test_stream_failure(self):
        sys.stdout.write("Output written to stdout\n")
        sys.stderr.write("Output written to stderr\n")
        fake_test = formatter.FakeTest('error_test')
        exc_info = self.makeExcInfo()
        self.subunit_formatter.test_failure(fake_test, 10, exc_info)
        self.restoreStreams()
        self.assertIn('STDOUT:', self.output.getvalue())
        self.assertIn('STDERR:', self.output.getvalue())

    def test_multiple_messages(self):
        # Only never-reported messages should be seen.
        fake_test = formatter.FakeTest('stdout_test')
        sys.stdout.write("First message written to stdout\n")
        sys.stderr.write("First message written to stderr\n")
        self.subunit_formatter.test_success(fake_test, 10)
        self.assertIn(
            "First message written to stdout", self.output.getvalue())
        self.assertIn(
            "First message written to stderr", self.output.getvalue())
        self.output.seek(0)
        self.output.truncate()
        sys.stdout.write("Second message written to stdout\n")
        sys.stderr.write("Second message written to stderr\n")
        self.subunit_formatter.test_success(fake_test, 10)
        self.restoreStreams()
        self.assertNotIn(
            "First message written to stdout", self.output.getvalue())
        self.assertNotIn(
            "First message written to stderr", self.output.getvalue())
