##############################################################################
#
# Copyright (c) 2004-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit tests for the testrunner's handling of non-unique test IDs."""


import unittest

from zope.testing import doctest
from zope.testing.testrunner.find import (
    DuplicateTestIDError,
    find_tests,
    tests_from_suite,
    )


class Options:
    """A basic mock of our command-line options."""
    keepbytecode = True
    at_level = 99
    test = []
    require_unique_ids = True


class UniquenessTestCase(unittest.TestCase):
    """A test case to test how the testrunner handles non-unique IDs."""

    def setUp(self):
        super(UniquenessTestCase, self).setUp()
        suites = [
            doctest.DocFileSuite('testrunner-ex/sampletests.txt'),
            doctest.DocFileSuite('testrunner-ex/sampletests.txt'),
            doctest.DocFileSuite('testrunner-ex/sampletestsl.txt'),
            doctest.DocFileSuite('testrunner-ex/sampletestsl.txt'),
            ]
        self.test_suites = unittest.TestSuite(suites)

    def _get_duplicate_tests(self, options):
        # We have to wrap this in a function as tests_from_suite()
        # returns a generator, so the error won't be triggered until we
        # actually try to iterate over the results.
        seen_test_ids = set()
        duplicated_test_ids = set()
        tests = [
            test for test in tests_from_suite(
                self.test_suites, options, seen_test_ids=seen_test_ids,
                duplicated_test_ids=duplicated_test_ids)]
        return duplicated_test_ids

    def test_tests_from_suite_records_duplicate_test_ids(self):
        """If tests_from_suite() encounters a test ID which has already
        been registered, it will record them in the duplicated_test_ids
        set it is passed.
        """
        duplicated_test_ids = self._get_duplicate_tests(
            Options())
        self.assertNotEqual(0, len(duplicated_test_ids))

    def test_tests_from_suite_ignores_duplicate_ids_if_option_not_set(self):
        """If the require_unique_ids flag is not set, tests_from_suite()
        will not record any of the IDs as duplicates.
        """
        options = Options()
        options.require_unique_ids = False
        duplicated_test_ids = self._get_duplicate_tests(
            options)
        self.assertEqual(0, len(duplicated_test_ids))

    def test_find_tests_raises_error_if_duplicates_found(self):
        """If find tests, which calls tests_from_suite, finds a
        duplicate test ID, it raises an error.
        """
        self.assertRaises(
            DuplicateTestIDError, find_tests, Options(), [self.test_suites])

    def test_DuplicateTestIDError_message_contains_all_test_ids(self):
        """The error raised by find_tests when duplicates are
        encountered contains all the duplicate test IDs it found.
        """
        try:
            find_tests(Options(), [self.test_suites])
        except DuplicateTestIDError, exception:
            pass
        self.assertIn('sampletests_txt', exception.message)
        self.assertIn('sampletestsl_txt', exception.message)
